#include "Display.h"
#include "SPI.h"
#include "Flappy.h"

#define GATE 25
#define GROUND 111//128-1-16
#define PIPE_WIDTH 16
#define PIPE_BORDER_HEIGHT 4
#define PIPE_BORDER_WIDTH 1
#define PIPE_REFLECTION 2
#define LCD_BUTTON_LEFT A4

Display lcd = Display();

void setup() {
  lcd.init(20);  //Display initialisieren
  pinMode(LCD_BUTTON_LEFT, INPUT);
  digitalWrite(LCD_BUTTON_LEFT, HIGH);
  lcd.clearVideoBuffer();
  drawGround();
  drawPipe(60, 50, true);
  drawPipe(40, 10, false);
  lcd.drawBitmap(16, 40, 16, 16, flappy);
  lcd.show();
}

void loop() {
  byte gateHeight = GATE;
  double speed = 0;
  double posY = 20;
  double time = 1;
  byte score = 0;
  boolean alive = false;
  delay(2000);
  while (digitalRead(LCD_BUTTON_LEFT)) {
  }
  alive = true;
  while(alive) {
    long gate = random(25, 80);
    gateHeight--; //lvl
    for (byte y=1; y<80; y++) {
      lcd.clearVideoBuffer();
      drawPipe(gate+gateHeight, y, true);
      drawPipe(gate-gateHeight, y, false);
      drawGround();
      if (y == 60) {
        score++;
      }
      
      if (( y > 60 && y < 80) && score > 0) {
        lcd.drawBitmap(16, 34, 16, 8, flappyFont+16*(score/10));
        lcd.drawBitmap(16, 24, 16, 8, flappyFont+16*(score%10));
        
      }
      time = time + 0.01;
      if (!digitalRead(LCD_BUTTON_LEFT)) {
        speed= 5;
        time=1;
      }

      posY -= speed * time;
      speed -= 1.0 * time; //1.0
      lcd.drawBitmap(posY, 40, 16, 16, flappy);

      //Collision
      if (posY > GROUND -16 || (y < 70 && y > 40 && posY > gate+gateHeight-16) || (y < 70 && y > 40 && posY < gate-gateHeight)) {
        alive = false;
        lcd.clearVideoBuffer();
        drawPipe(gate+gateHeight, y, true);
        drawPipe(gate-gateHeight, y, false);
        drawGround();
        lcd.drawBitmap(posY, 40, 16, 16, flappyRIP);
        lcd.drawBitmap(16, 34, 16, 8, flappyFont+16*(score/10));
        lcd.drawBitmap(16, 24, 16, 8, flappyFont+16*(score%10));
        lcd.show();
        break;
      }
      lcd.show();
      delay(50);
    }
  }
}

void drawGround() {
  lcd.drawLine(GROUND, 0, GROUND, Display::HEIGHT);
  lcd.drawLine(GROUND+3, 0, GROUND+3, Display::HEIGHT);
}

void drawPipe(byte x, byte y, boolean ground) {
  byte pipeBorderX = ground ? x+PIPE_BORDER_HEIGHT : x-PIPE_BORDER_HEIGHT;//xposition Rand
  byte yPipeWidth = y-PIPE_WIDTH < 0 ? 0 : y-PIPE_WIDTH;
  lcd.drawLine(x, y, x, yPipeWidth);//Röhreneingang
  lcd.drawLine(pipeBorderX, y, pipeBorderX, yPipeWidth);//Röhrenrand
  lcd.drawLine(x, y, ground ? x+PIPE_BORDER_HEIGHT : x-PIPE_BORDER_HEIGHT, y);//vorderes Seitenteil Röhrenrand
  lcd.drawLine(x, y-PIPE_REFLECTION, ground ? x+PIPE_BORDER_HEIGHT : x-PIPE_BORDER_HEIGHT, y-PIPE_REFLECTION);//vorderes Seitenteil Röhrenrand Reflektion
  lcd.drawLine(x, y-PIPE_WIDTH, ground ? x+PIPE_BORDER_HEIGHT : x-PIPE_BORDER_HEIGHT, y-PIPE_WIDTH);//hinteres Seitenteil Röhrenrand
  lcd.drawLine(pipeBorderX, y-PIPE_BORDER_WIDTH, ground ? GROUND : 0, y-PIPE_BORDER_WIDTH);//Vorderseite
  lcd.drawLine(pipeBorderX, y-PIPE_WIDTH+PIPE_BORDER_WIDTH, ground ? GROUND : 0, y-PIPE_WIDTH+PIPE_BORDER_WIDTH);//Hinterseite
  lcd.drawLine(pipeBorderX, y-PIPE_REFLECTION-PIPE_BORDER_WIDTH, ground ? GROUND : 0, y-PIPE_REFLECTION-PIPE_BORDER_WIDTH);//Reflektionslinie
}








